<?php

namespace modules\crm\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Images;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use modules\crm\Models\BusinessContact;
use modules\crm\Models\BusinessContactDocument;
use modules\crm\Models\BusinessContactTransaction;
use modules\crm\Models\CustomerGroup;
use modules\crm\services\CrmService;
use modules\location\Models\Location;

class CrmController extends Controller{

    public function index(Request $request){
        $search = [];
        #dd(auth()->user()->getAllPermissions());
        
        if(isset($request->criteria)){
            $search = [                
                'criteria' => $request->criteria, 
                'keyword' => $request->keyword
            ];            
        }
                
        return view("crm::index", [            
            'search' => $search
        ]);
    }
    
    public function create(String $type, Request $request){
        if($request->ajax()){
            $groups = [];
            if($type == 'customer'){
                $groups = CustomerGroup::all();
            }

            $sites = Location::where('type', 'site')->get();
            
            return response()->json([
                'groups' => $groups,
                'sites' => $sites
            ]);
        }
        
        return view("crm::create");
    }

    public function store(Request $request)
    {
        $group = json_decode($request->group);
        
        // Validation des données
        #dd($request->all());
        $this->validateRequest($request, $group);

        try {
            DB::beginTransaction();

            // Génération de la référence unique
            $category = $request->type;
            $numero = $this->generateReference($category);

            // Décodage des objets JSON issus de la requête
            $status = json_decode($request->status);
            $company = json_decode($request->company);

            // Détermination du nom de contact en fonction du type de groupe (B2B ou non)
            $lastname = $category == 'customer' ? ((int)$group->b2b == 1 ? $company->contact->lastname : $request->lastname) : $request->company_name;

            // Création du contact d'affaires
            $insertdata = [
                'location_id' => null,
                'category' => $category,
                'group_id' => $group->id ?? null,
                'group' => $group->title ?? null,
                'numero' => $numero['numero'],
                'company' => $company->name ?? null,
                'reference' => $numero['reference'],
                'lastname' => $lastname,
                'firstname' => $category == 'customer' ? ((int) $group->b2b == 1 ? $company->contact->firstname : $request->firstname) : '',
                'address' => $request->address,
                'postcode' => $request->postcode,
                'city' => $request->city,
                'email' => $request->email,
                'phone' => $request->phone,
                'status' => $status->value,
                'contact_phone' => $company->contact->phone ?? null,
                'bank_info' => $company->bank ?? null,
                'company_info' => $company->info ?? null,
                'credit' => (bool)$request->credit,
                'userid' => Auth::user()->id,
                'username' => Auth::user()->name,
                'created_at' => date_now(),
                'updated_at' => date_now(),
            ];            
            #dd($insertdata);
            $businessContact = BusinessContact::create($insertdata);

            // Génération du slug
            $slug = $businessContact->id . '-' . strtolower(generateString(12));
            $businessContact->slug = $slug;

            // Gestion de l'upload d'avatar
            $this->handleAvatarUpload($request, $businessContact, $category);

            // Sauvegarde des informations du contact
            $businessContact->save();

            // Insertion du crédit dans le compte client si applicable
            $this->insertCredit($businessContact, $request, $lastname, $category);

            DB::commit();

            return response()->json([
                'status' => 'success',
                'message' => message()['success'],
                'detail' => [
                    'data' => $businessContact,
                    'url' => route('app.crm.detail', ['slug' => $slug, 'type' => $category])
                ]
            ]);
        } catch (\Throwable $exception) {
            DB::rollBack();
            throw $exception;
        }
    }


    private function validateRequest(Request $request, $group)
    {        
        #dd($request->all());
        if($request->type == 'customer'){
            if ((int) $group->b2b == 1) {
                return $request->validate([
                    'company_name' => 'required',
                    'company_contact_lastname' => 'required',
                    'phone' => 'required',
                    #'site_id' => 'required'
                ]);
            } else {
                return $request->validate([
                    'lastname' => 'required',
                    'phone' => 'required',
                    #'site_id' => 'required'
                ]);
            }
        }else{
            return $request->validate([
                'company_name' => 'required',
                'phone' => 'required'
            ]);
        }
    }

    private function generateReference($category)
    {
        $lastEntry = BusinessContact::whereNotNull('numero')->where('category', $category)->orderByDesc('created_at')->first();
        $lastNumber = $lastEntry ? $lastEntry->numero : 0;
        $newNumero = $lastNumber + 1;

        return [
            'numero' => $newNumero,
            'reference' => auto_number($category, $newNumero)
        ];
    }

    private function handleAvatarUpload(Request $request, BusinessContact $businessContact, $category)
    {
        if ($request->hasFile('file')) {
            $folder = "images/business_contacts/{$category}/avatar";
            $dataFile = $request->file;

            if ($dataFile->isValid()) {
                $image = new Images($folder);
                $uploadResponse = $image->image_upload($dataFile, ['width' => '300px', 'height' => '300px']);
                $businessContact->avatar = $uploadResponse['filename'];
            }
        }
    }

    private function insertCredit($businessContact, $request, $lastname, $category)
    {
        if (isset($request->credit) && (float) convertToFloat($request->rap) > 0) {
            BusinessContactTransaction::create([
                'business_contact_id' => $businessContact->id,
                'business_contact_name' => $lastname,
                'business_contact_type' => $category,
                'transaction_type' => 'out',
                'amount' => $request->rap * -1,
                'description' => 'RAP antérieur',
                'balance_after_transaction' => 0, #$request->rap * -1,
                'created_at' => date_now(),
                'updated_at' => date_now(),
            ]);
        }
    }

    
    public function detail(String $type, String $slug, Request $request){
        if($request->ajax()){
            $res = BusinessContact::with('group', 'location')->where('slug', $slug);
            switch ($request->tab) {
                case 'document':
                    $business_contact = $res->with('documents')->first();
                    break;            
                default:
                    $business_contact = $res->first();
                    break;
            }


            $groups = [];
            if($type == 'customer'){
                $groups = CustomerGroup::select('id', 'title', 'b2b', 'credit', 'rate_type', 'rate_value')->get();
            }

            $sites = Location::select('id', 'title', 'type')->where('type', 'site')->get();     

            return response()->json([
                'status' => 'success',
                'business_contact' => $business_contact,
                'groups' => $groups,
                'sites' => $sites
            ]);
        }
        
        return view("crm::detail");
    }

    public function info(Request $request) {
        try{
            #dd($request->all());
            $request->validate([
                "lastname" => "required"
            ]);
            $id = $request->id;         
            
            $group = json_decode($request->group); 

            //Décodage des objets JSON issus de la requête
            $status = json_decode($request->status);
            $company = json_decode($request->company);

            //Détermination du nom de contact en fonction du type de groupe (B2B ou non)
            $category = $request->category;
            $firstname = null;
            if($category == 'customer'){
                $lastname = (int) $group->b2b == 1 ? $company->contact->lastname : $request->lastname;
                $firstname = (int) $group->b2b == 1 ? $company->contact->firstname : $request->firstname;
            }else{
                $lastname = $company->name;
            }

            $updatedata = [
                'location_id' => null,#$request->site_id,
                'group_id' => $category == 'customer' ? $group->id : null,
                'group' => $category == 'customer' ? $group->title : null,
                'company' => $company->name ?? null,                
                'lastname' => $lastname,
                'firstname' => $firstname,
                'address' => $request->address,
                'postcode' => $request->postcode,
                'city' => $request->city,
                'email' => $request->email,
                'phone' => $request->phone,                
                'contact_phone' => $company->contact->phone ?? null,                
                'credit' => (bool)$request->credit,
                "updated_at" => date_now(),
                "update_by" => json_encode([
                    'user' => Auth::user()->id,
                    'name' => Auth::user()->id
                ])
            ];  
            BusinessContact::where("id", $id)->update($updatedata);

            $res = BusinessContact::with('group', 'location')->where('id', $id)->first();
            return response()->json([
                'status' => 'success',
                'message' => message()['update'],
                'business_contact' => $res
            ]);
        }catch(Exception $ex){
            throw $ex;
        }
    }    

    public function fieldUpdate(Request $request){
        try{            
            $request->validate([                
                'id' => 'required',
            ]);
            
            $id = $request->id;
            switch ($request->field) {
                case 'groupe':
                    $fields = [
                        "group_id" => $request->group['id'],
                        "group" => $request->group['title'],
                    ];
                    break; 
                case 'status':
                        $fields = [
                            "status" => !$request->status ? 'active' : 'inactive'
                        ];
                        break;                
                default:
                    $fields[] = $request->field;
                    break;
            }            


            foreach($fields as $field => $value){
                BusinessContact::where('id', $id)->update([
                    $field => $value,
                    "updated_at" => date_now(),
                    "update_by" => json_encode([
                        'user' => Auth::user()->name,
                        'date' => date_now()
                    ])
                ]);
            }            

            return response()->json([
                'status' => 'success',
                'message' => message()['update'],
                'business_contact' => BusinessContact::where('id', $id)->first()
            ]);
        }catch(Exception $ex){
            throw $ex;            
        }
    }

    public function destroy(string $type, int $id){        
        BusinessContact::find($id)->delete();
        return response()->json([
            'status' => 'success',
            'message' => 'Revendeur supprimé avec succès'
        ]);
    }

    public function logo(Request $request){ 
        $request->validate([
            'file' => 'required'
        ]);        

        $id = $request->id;
        $business_contact = BusinessContact::find($id);
        
        if ($request->hasFile("file")) {
            $folder = "images/business_contacts/avatar";
            $datafile = $request->file;                

            if ($datafile->isValid()) {
                $img = new Images($folder);
                $rep = $img->image_upload($datafile, ['width' => '300px', 'height' => '300px']);
                $business_contact->avatar = $rep['filename'];
            }
        }
        $business_contact->save();

        return response()->json([
            'status' => 'success',
            'message' => message()['update'],
            'business_contact' => BusinessContact::find($id)
        ]);
    }

    public function documents(Request $request)
    {
        try {
            #dd($request->all());
            $request->validate([
                'title' => 'required',
                'file' => 'required'
            ]);

            DB::beginTransaction();

            $business_contact_id = $request->id;
            $business_contact = BusinessContact::find($business_contact_id);
            $doc = BusinessContactDocument::create([
                "business_contact_id" => $business_contact_id,
                "title" => $request->title,
                "comments" => $request->comments,
                "created_at" => date_now(),
                "updated_at" => date_now(),
            ]);            

            if ($request->hasFile("file")) {
                $folder = "documents/business_contacts/". $business_contact->slug;
                $datafile = $request->file;
                $fileType = $datafile->getMimeType();                

                if ($datafile->isValid()) {
                    $file = new Images($folder);
                    $rep = $file->move_file($datafile);                    
                    
                    BusinessContactDocument::where("id", $doc->id)->update([
                        'filename' => $rep['filename'],
                        'type' => $fileType,
                        'folder' => $business_contact->slug
                    ]);
                }
            }

            DB::commit();
            
            $business_contact = BusinessContact::find($business_contact_id)->with('documents')->first();
            return response()->json([
                'status' => 'success',
                'message' => message()['success'],
                'business_contact' => $business_contact
            ]);
        } catch (Exception $ex) {
            DB::rollBack();
            throw $ex;
        }
    }

    public function document_destroy($id){
        $doc = BusinessContactDocument::find($id);        
        
        $folder = asset('documents/business_contacts/'.$doc->folder.'/'.$doc->filename);
        if(File::exists($folder)){
            File::delete($folder);
        }
        $doc->delete();

        $business_contact_id = $doc->business_contact_id;
        $business_contact = BusinessContact::where('id', $business_contact_id)->with('documents')->first();
        
        return response()->json([
            'status' => 'success',
            'message' => 'Document supprimé avec succès',
            'business_contact' => $business_contact
        ]);
    }

    public function status(Request $request){        
        $id = $request->id;
        $status = $request->status['value'];
        $business_contact = BusinessContact::find($id);
        $business_contact->status = $status;
        $business_contact->save();

        return response()->json([
            'status' => 'success',
            'message' => message()['update'],
            'business_contact' => BusinessContact::find($id)
        ]);
    }

    public function data(Request $request){
        $type = $request->type;       
        $page = isset($request->pagesize) ? $request->pagesize : 25;
        if($request->has('data') && $request->data == 'all'){
            $page = 0;
        }

        $filter = [];
        if(isset($request->criteria)){
            $criteria = $request->criteria;
            if($criteria == 'reference'){
                $filter['reference'] = $request->keyword;
            }
            if($criteria == 'name'){
                $filter['name'] = $request->keyword;
            }
            if($criteria == 'email'){
                $filter['email'] = $request->keyword;
            }
            if($criteria == 'phone'){
                $filter['phone'] = $request->keyword;
            }
        }
        
        if ($type == 'customer') {            
            $businessContacts = CrmService::customers($filter, $page);
        }        
        if ($type == 'supplier') {            
            $businessContacts = CrmService::suppliers($filter, $page);
        }
                
        $permissions = array_values(auth()->user()->getAllPermissions()->filter(function ($permission) {
            return str_contains($permission->name, request()->type);
        })->toArray());

        return response()->json([
            'status' => 'success',
            'businessContacts' => $businessContacts,
            'permissions' => $permissions
        ]);
    }
    
    public function customers(Request $request){
        /*$location = my_location();
        if($request->has('location')){
            $location = Location::find($request->location);
        }*/        
        #dd($request->type);
        if($request->type == 'customer'){
            #dd($location);
            $businessContacts = CrmService::customers([
                #'location' => $location->id
            ]);
        }else{
            $businessContacts = Location::where([
                'stock_management' => true,
                'is_parent' => false
            ])->get();
        }        

        return response()->json([
            'status' => 'success',
            'businessContacts' => $businessContacts,
        ]);
    }



    
}