<?php

namespace modules\crm\services;

use Illuminate\Support\Arr;
use modules\crm\Models\BusinessContact;
use modules\crm\Models\BusinessContactTransaction;
use modules\crm\Models\CustomerGroup;
use modules\location\Models\Location;
use modules\sales\Models\AccountingDocument;
use modules\sales\Models\AccountingDocumentPayment;
use modules\sales\Models\Receipt;
use modules\sales\Models\ReceiptPayment;
use modules\sales\services\AccountingDocService;

class CrmService
{
    public static function customers(array $filters = [], int $pagination = 0, array $with = [])
    {
        $filters['category'] = 'customer';
        return self::getdata($filters, $pagination, $with);
    }    
    
    public static function suppliers(array $filters = [], int $pagination = 0)
    {        
        $filters['category'] = 'supplier';
        return self::getdata($filters, $pagination);
    }

    public function oldtransactions(int $business_contact, int $page) {
        $transactions = BusinessContactTransaction::where('business_contact_id', $business_contact)
            ->orderByDesc('created_at')
            ->paginate($page);
    
        // Récupérer tous les documents liés aux transactions en une seule requête
        $documentIds = $transactions->pluck('document_id')->unique();
        $documents = AccountingDocument::whereIn('id', $documentIds)
                        ->get(['id', 'location_id', 'total_ttc'])
                        ->keyBy('id');
    
        // Récupérer tous les paiements et les regrouper par document
        $payments = AccountingDocumentPayment::whereIn('accounting_document_id', $documentIds)
                        ->selectRaw('accounting_document_id, SUM(amount) as total_paid')
                        ->groupBy('accounting_document_id')
                        ->pluck('total_paid', 'accounting_document_id'); // Clé = document_id, Valeur = somme des paiements
    
        // Filtrer les transactions
        $filtered = $transactions->getCollection()->filter(function ($item) use ($documents, $payments) {
            $document = $documents[$item->document_id] ?? null;
            if (!$document) return false;

            // Calcul du RAP (reste à payer)
            $totalPaid = $payments[$item->document_id] ?? 0;
            $rap = $document->total_ttc - $totalPaid;

            return $rap != 0; // Exclure les documents payés
        })->map(function ($item) use ($documents) {
            $document = $documents[$item->document_id] ?? null;
            $location = $document ? Location::find($document->location_id) : null;

            return [
                'id' => $item->id,
                'location' => $location,
                'date' => date('d-m-Y', strtotime($item->created_at)),
                'type' => (new AccountingDocService())->documenttype()[$item->document_type] ?? 'Inconnu',
                'reference' => $item->document_reference,
                'credit' => $item->transaction_type == 'in' ? $item->amount : 0,
                'debit' => $item->transaction_type == 'out' ? $item->amount : 0,
                'payment_method' => $item->payment_method,
                'description' => $item->description
            ];
        })->values(); // Réindexer la collection
        
        #dd($filtered->toArray());

    
        // Calcul des totaux
        $totalCredit = $filtered->sum('credit');
        $totalDebit = $filtered->sum('debit');
        $balance = $totalCredit - $totalDebit;
    
        // Appliquer la nouvelle collection filtrée sur l'objet de pagination
        $transactions->setCollection($filtered);
    
        return [
            'transactions' => $transactions,
            'total_credit' => $totalCredit,
            'total_debit' => $totalDebit,
            'total_payment' => $payments->sum(),
            'balance' => $balance
        ];
    }    

    public function transactions(int $business_contact, int $page) {
        $transactions = BusinessContactTransaction::where('business_contact_id', $business_contact)
            ->orderByDesc('created_at')
            ->paginate($page);
    
        // Récupérer tous les document_id et receipt_id
        $documentIds = $transactions->pluck('document_id')->unique()->filter();
        $receiptIds = $transactions->pluck('receipt_id')->unique()->filter();
    
        // Récupérer les documents et les reçus
        $documents = AccountingDocument::whereIn('id', $documentIds)
                        ->get(['id', 'location_id', 'total_ttc'])
                        ->keyBy('id');
    
        $receipts = Receipt::whereIn('id', $receiptIds)
                        ->get(['id', 'location_id', 'total_ttc'])
                        ->keyBy('id');
    
        // Récupérer les paiements des documents
        $documentPayments = AccountingDocumentPayment::whereIn('accounting_document_id', $documentIds)
                            ->selectRaw('accounting_document_id, SUM(amount) as total_paid')
                            ->groupBy('accounting_document_id')
                            ->pluck('total_paid', 'accounting_document_id');
    
        // Récupérer les paiements des reçus
        $receiptPayments = ReceiptPayment::whereIn('receipt_id', $receiptIds)
                            ->selectRaw('receipt_id, SUM(amount) as total_paid')
                            ->groupBy('receipt_id')
                            ->pluck('total_paid', 'receipt_id');
    
        // Fusionner les paiements
        $payments = $documentPayments->merge($receiptPayments);
    
        // Filtrer les transactions
        $filtered = $transactions->getCollection()->filter(function ($item) use ($documents, $receipts, $payments) {
            // Vérifier si c'est un document ou un reçu
            $document = $documents[$item->document_id] ?? null;
            $receipt = $receipts[$item->receipt_id] ?? null;
    
            if (!$document && !$receipt) return false;
    
            // Calcul du RAP (reste à payer)
            $totalPaid = $payments[$item->document_id] ?? $payments[$item->receipt_id] ?? 0;
            $totalTTC = $document->total_ttc ?? $receipt->total_ttc ?? 0;
            $rap = $totalTTC - $totalPaid;
    
            return $rap != 0; // Exclure les documents/reçus payés
        })->map(function ($item) use ($documents, $receipts) {
            $document = $documents[$item->document_id] ?? null;
            $receipt = $receipts[$item->receipt_id] ?? null;
    
            $location = $document ? Location::find($document->location_id) : ($receipt ? Location::find($receipt->location_id) : null);
    
            return [
                'id' => $item->id,
                'location' => $location,
                'date' => date('d-m-Y', strtotime($item->created_at)),
                'type' => (new AccountingDocService())->documenttype()[$item->document_type] ?? 'Ticket de caisse',
                'reference' => $item->document_reference,
                'credit' => $item->transaction_type == 'in' ? $item->amount : 0,
                'debit' => $item->transaction_type == 'out' ? $item->amount : 0,
                'payment_method' => $item->payment_method,
                'description' => $item->description
            ];
        })->values();
    
        // Calcul des totaux
        $totalCredit = $filtered->sum('credit');
        $totalDebit = $filtered->sum('debit');
        $balance = $totalCredit - $totalDebit;
    
        // Appliquer la nouvelle collection filtrée sur l'objet de pagination
        $transactions->setCollection($filtered);
    
        return [
            'transactions' => $transactions,
            'total_credit' => $totalCredit,
            'total_debit' => $totalDebit,
            'total_payment' => $payments->sum(),
            'balance' => $balance
        ];
    }
    

    protected static function getdata(array $filters = [], int $pagination = 0, array $with = []){        
        $query = BusinessContact::query();
        $query->with($with);
        $query->withSum(['transactions as total_in' => function ($q) {
                    $q->where('transaction_type', 'in');
                }], 'document_amount')
                ->withSum(['transactions as total_out' => function ($q) {
                    $q->where('transaction_type', 'out');
                }], 'document_amount')
                ->withSum(['accounting_documents as total_invoice' => function ($q) {
                    $q->where('document_type', 'invoice');
                }], 'total_ttc')
                ->withSum(['accounting_documents as total_credit' => function ($q) {
                    $q->where('document_type', 'credit');
                }], 'total_ttc');

        if (!empty($filters)) {                            
            self::applyFilters($query, $filters);
        }        
        
        $customers = $pagination == 0 ? $query->get() : $query->paginate($pagination);
        #dd($customers, $filters);
        
        $customers->transform(function($customer) use ($filters){
            $customer = $customer->toArray();
            
            if(isset($filters['category']) and $filters['category'] == 'customer'){
                $name = (int)$customer['group']['b2b'] == 1 ? $customer['company'] : $customer['lastname'] . ' ' . $customer['firstname'];
            }else{
                $name = !is_null($customer['company']) ? $customer['company'] : $customer['lastname'] . ' ' . $customer['firstname'];
            }

            $customer['title'] = $name;

            $customer['invoice'] = [];
            $customer['order'] = [];
            $customer['delivery'] = [];
            $customer['return'] = [];
            $customer['quote'] = [];
            $customer['rap'] = ($customer['total_in'] ?? 0) - ($customer['total_out'] ?? 0);

            #dd($customer, $name);

            return $customer;
        });           
        
        #dd($customers->toArray(), $filters);
        
        return $customers;
    }

    // Appliquer les filtres de recherche
    private static function applyFilters($query, array $filters)
    {
        if(isset($filters['category'])){
            if($filters['category'] == 'customer'){
                $query->with(['group']);
            }
            $query->where('category', $filters['category']);
        }
        if (isset($filters['id'])) {
            $query->where('business_contacts.id', $filters['id']);
        }
        if (isset($filters['slug'])) {
            $query->where('slug', $filters['slug']);
        }        
        if (isset($filters['location'])) {
            $query->where('location_id', $filters['location']);
        }        
        if (isset($filters['group_id'])) {
            $query->where('group_id', $filters['group_id']);
        }        

        if (isset($filters['keyword']) && isset($filters['criteria'])) {
            $query->where("products.{$filters['criteria']}", 'LIKE', "%{$filters['keyword']}%");
        }

        if(isset($filters['reference'])){
            $query->where('reference', $filters['reference']);
        }
        if(isset($filters['name'])){
            $query->where('lastname', "LIKE", "%".$filters['name']."%")
                    ->orWhere('firstname', "LIKE", "%".$filters['name']."%")
                    ->orWhere('company', "LIKE", "%".$filters['name']."%");

        }
        if(isset($filters['email'])){
            $query->where('email', $filters['email']);
        }
        if(isset($filters['phone'])){
            $query->where('phone', $filters['phone']);
        }
    }
}